char *var_smtp_host_lookup;
char *var_smtpd_ehlo_dis_words;
char *var_milt_head_checks;
char *var_milt_macro_deflts;
char *var_smtpd_tls_eecdh;
char *var_scache_service;
char *var_smtpd_sasl_tls_opts;
char *var_virt_alias_doms;
char *var_psc_forbid_cmds;
char *var_virt_mailbox_base;
char *var_smtpd_tls_CApath;
char *var_smtp_tls_ciph;
char *var_cleanup_milters;
char *var_verify_map;
char *var_tlsp_tls_mand_excl;
char *var_smtpd_tls_proto;
char *var_smtpd_null_key;
char *var_rcpt_canon_maps;
char *var_full_name_encoding_charset;
char *var_local_rcpt_maps;
char *var_msg_strip_chars;
char *var_smtp_tcp_port;
char *var_local_cmd_shell;
char *var_smtpd_tls_eckey_file;
char *var_newalias_path;
char *var_tlsp_clnt_CApath;
char *var_tls_rand_exch_name;
char *var_smtpd_tls_ciph;
char *var_smtpd_milter_maps;
char *var_xforward_hosts;
char *var_smtp_head_chks;
char *var_smtp_pix_bug_maps;
char *var_bounce_rcpt;
char *var_vrfy_virt_xport;
char *var_psc_barlf_action;
char *var_null_def_xport_maps_key;
char *var_tlsp_tls_proto;
char *var_unv_rcpt_tf_act;
char *var_smtpd_proxy_opts;
char *var_milt_helo_macros;
char *var_cmd_exp_filter;
char *var_delay_rcpt;
char *var_smtp_sasl_passwd;
char *var_virt_mailbox_maps;
char *var_tlsp_tls_mand_proto;
char *var_mail_spool_dir;
char *var_tlsp_tls_dcert_file;
char *var_smtpd_sasl_realm;
char *var_smtp_mime_chks;
char *var_smtp_sasl_tls_opts;
char *var_qmqpd_clients;
char *var_vrfy_def_xport;
char *var_tlsp_tls_chain_files;
char *var_psc_dnsbl_action;
char *var_smtpd_tls_dh1024_param_file;
char *var_def_transport;
char *var_html_dir;
char *var_smtp_sasl_opts;
char *var_smtpd_forbid_bare_lf_excl;
char *var_masq_domains;
char *var_fwd_exp_filter;
char *var_virt_mailbox_lock;
char *var_tlsp_tls_fpt_dgst;
char *var_smtp_tls_key_file;
char *var_flush_acl;
char *var_milt_rcpt_macros;
char *var_canon_classes;
char *var_local_login_snd_maps;
char *var_tlsp_tls_eecdh;
char *var_data_checks;
char *var_smtp_tls_mand_proto;
char *var_mimehdr_checks;
char *var_allow_commands;
char *var_smtp_tls_dcert_file;
char *var_smtp_sasl_passwd_res_delim;
char *var_master_disable;
char *var_psc_allist_if;
char *var_readme_dir;
char *var_tlsp_clnt_level;
char *var_tlsp_clnt_fpt_dgst;
char *var_tlsp_tls_loglevel;
char *var_tlsp_tls_ciph;
char *var_lmtp_tls_loglevel;
char *var_body_checks;
char *var_smtpd_acl_perm_log;
char *var_smtp_tlsrpt_sockname;
char *var_multi_start_cmds;
char *var_tlsp_tls_dh1024_param_file;
char *var_lmtp_tls_scache_db;
char *var_smtp_tls_CApath;
char *var_bestmx_transp;
char *var_smtpd_sasl_exceptions_networks;
char *var_milt_daemon_name;
char *var_proxy_write_maps;
char *var_virt_dsn_filter;
char *var_tlsp_tls_eccert_file;
char *var_smtpd_tls_mand_ciph;
char *var_header_checks;
char *var_relocated_maps;
char *var_psc_pregr_banner;
char *var_tlsp_clnt_loglevel;
char *var_smtpd_tls_scache_db;
char *var_smtp_tls_fpt_cmatch;
char *var_smtpd_sasl_opts;
char *var_allow_files;
char *var_verify_sender;
char *var_smtpd_tls_cert_file;
char *var_virt_uid_maps;
char *var_smtp_tls_per_site;
char *var_showq_acl;
char *var_tlsp_tls_CApath;
char *var_smtp_pix_bug_words;
char *var_smtp_tls_tafile;
char *var_remote_rwr_domain;
char *var_smtp_sasl_path;
char *var_smtp_resp_filter;
char *var_smtp_generic_maps;
char *var_notify_classes;
char *var_hfrom_format;
char *var_defer_xports;
char *var_empty_addr;
char *var_send_bcc_maps;
char *var_fallback_relay;
char *var_relay_transport;
char *var_smtp_tls_level;
char *var_tlsp_clnt_dcert_file;
char *var_dnsblog_service;
char *var_smtp_tls_vfy_cmatch;
char *var_nesthdr_checks;
char *var_xclient_hosts;
char *var_cyrus_conf_path;
char *var_smtp_tls_sec_cmatch;
char *var_smtp_tls_mand_ciph;
char *var_sm_fix_eol;
char *var_smtp_tls_scache_db;
char *var_smtpd_sasl_service;
char *var_smtp_sasl_type;
char *var_psc_uproxy_proto;
char *var_tlsp_tls_mand_ciph;
char *var_nbdb_allow_user_pfxs;
char *var_smtpd_sasl_mech_filter;
char *var_smtpd_cmd_filter;
char *var_proxy_read_maps;
char *var_psc_dnsbl_reply;
char *var_tlsp_tls_eckey_file;
char *var_smtpd_policy_context;
char *var_etrn_checks;
char *var_smtp_bind_addr6;
char *var_conc_neg_feedback;
char *var_masq_classes;
char *var_smtp_tls_cert_file;
char *var_smtpd_forbid_bare_lf;
char *var_smtpd_tls_CAfile;
char *var_smtpd_sasl_path;
char *var_smtp_ehlo_dis_words;
char *var_null_local_login_snd_maps_key;
char *var_milt_eoh_macros;
char *var_smtp_reqtls_policy;
char *var_filter_xport;
char *var_vrfy_snd_def_xport_maps;
char *var_tlsp_tls_cert_file;
char *var_tls_rand_source;
char *var_local_rwr_clients;
char *var_smtp_addr_pref;
char *var_smtp_bind_addr;
char *var_smtp_sasl_mechs;
char *var_unv_from_tf_act;
char *var_perm_mx_networks;
char *var_psc_rej_ftr_maps;
char *var_rcpt_witheld;
char *var_psc_dnsbl_sites;
char *var_smtpd_tls_level;
char *var_maps_rbl_domains;
char *var_stress;
char *var_milt_def_action;
char *var_smtpd_noop_cmds;
char *var_milt_eod_macros;
char *var_unk_addr_tf_act;
char *var_smtp_tls_eckey_file;
char *var_tlsp_clnt_cert_file;
char *var_smtpd_sasl_type;
char *var_smtp_sasl_auth_cache_name;
char *var_rcpt_bcc_maps;
char *var_rbl_reply_maps;
char *var_local_transport;
char *var_always_bcc;
char *var_smtpd_tls_dkey_file;
char *var_smtpd_milters;
char *var_smtpd_tls_key_file;
char *var_virt_alias_maps;
char *var_smtpd_proxy_filt;
char *var_verp_clients;
char *var_mailq_path;
char *var_milt_unk_macros;
char *var_milt_conn_macros;
char *var_smtp_tls_chain_files;
char *var_unk_name_tf_act;
char *var_nbdb_allow_root_pfxs;
char *var_multi_cntrl_cmds;
char *var_unv_from_why;
char *var_smtpd_tls_eccert_file;
char *var_mail_checks;
char *var_sendmail_path;
char *var_psc_pipel_action;
char *var_milt_data_macros;
char *var_smtpd_service;
char *var_smtpd_tls_chain_files;
char *var_error_rcpt;
char *var_home_mailbox;
char *var_smtp_body_chks;
char *var_milt_protocol;
char *var_snd_def_xport_maps;
char *var_vrfy_relay_xport;
char *var_tlsp_clnt_CAfile;
char *var_smtpd_dns_re_filter;
char *var_smtp_helo_name;
char *var_virt_mailbox_doms;
char *var_smtp_tls_fpt_dgst;
char *var_pipe_dsn_filter;
char *var_reject_tmpf_act;
char *var_smtpd_relay_ccerts;
char *var_smtp_dns_support;
char *var_alias_maps;
char *var_eod_checks;
char *var_smtp_cache_dest;
char *var_smtp_tls_CAfile;
char *var_smtp_sasl_tlsv_opts;
char *var_psc_cmd_filter;
char *var_tlsp_clnt_eckey_file;
char *var_smtp_tls_insecure_mx_policy;
char *var_smtp_tls_loglevel;
char *var_tlsp_tls_level;
char *var_smtpd_tls_excl_ciph;
char *var_smtp_tls_dkey_file;
char *var_milt_v;
char *var_tlsp_clnt_policy;
char *var_smtp_tls_eccert_file;
char *var_smtpd_snd_auth_maps;
char *var_psc_cache_map;
char *var_smtp_nest_chks;
char *var_tlsp_tls_dkey_file;
char *var_smtp_dsn_filter;
char *var_smtp_tls_sni;
char *var_canonical_maps;
char *var_null_relay_maps_key;
char *var_vrfy_relayhost;
char *var_tlsp_tls_CAfile;
char *var_smtpd_hoggers;
char *var_rcpt_checks;
char *var_smtpd_ehlo_dis_maps;
char *var_vrfy_xport_maps;
char *var_virt_gid_maps;
char *var_client_checks;
char *var_smtp_dns_re_filter;
char *var_virt_transport;
char *var_deliver_hdr;
char *var_manpage_dir;
char *var_fallback_transport;
char *var_tlsp_clnt_dkey_file;
char *var_smtpd_uproxy_proto;
char *var_helo_checks;
char *var_local_dsn_filter;
char *var_rcpt_canon_classes;
char *var_msg_reject_chars;
char *var_submit_acl;
char *var_prop_extension;
char *var_masq_exceptions;
char *var_relay_rcpt_maps;
char *var_use_srv_lookup;
char *var_smtp_tls_excl_ciph;
char *var_mailbox_lock;
char *var_smtpd_banner;
char *var_rest_classes;
char *var_tlsp_tls_key_file;
char *var_smtpd_tls_mand_excl;
char *var_smtpd_tls_mand_proto;
char *var_sample_dir;
char *var_tlsp_tls_excl_ciph;
char *var_smtpd_tls_dcert_file;
char *var_psc_nsmtp_action;
char *var_vrfy_local_xport;
char *var_tlsp_clnt_per_site;
char *var_tlsp_tls_dh512_param_file;
char *var_def_filter_nexthop;
char *var_multi_stop_cmds;
char *var_tlsp_clnt_key_file;
char *var_smtp_tls_policy;
char *var_smtp_ehlo_dis_maps;
char *var_smtpd_forbid_cmds;
char *var_smtpd_tls_dh512_param_file;
char *var_psc_pregr_action;
char *var_psc_ehlo_dis_words;
char *var_vrfy_relay_maps;
char *var_mailbox_cmd_maps;
char *var_non_empty_eoh_action;
char *var_psc_tls_level;
char *var_2bounce_rcpt;
char *var_psc_ehlo_dis_maps;
char *var_tlsp_clnt_eccert_file;
char *var_mbox_transp_maps;
char *var_tlsp_clnt_chain_files;
char *var_smtp_tls_mand_excl;
char *var_send_canon_maps;
char *var_snd_relay_maps;
char *var_send_canon_classes;
char *var_smtp_vrfy_tgt;
char *var_bounce_tmpl;
char *var_smtpd_rej_ftr_maps;
char *var_psc_acl;
char *var_input_transp;
char *var_smtp_tls_proto;
char *var_milt_mail_macros;
char *var_conc_pos_feedback;
char *var_relay_checks;
char *var_smtpd_tls_fpt_dgst;
char *var_tlsproxy_service;
char *var_fbck_transp_maps;
char *var_exec_exp_filter;
char *var_smtpd_reject_filter_maps;
char *var_smtpd_policy_def_action;
char *var_transport_maps;
char *var_mailbox_transport;
char *var_unv_rcpt_why;
char *var_smtpd_proxy_ehlo;
char *var_smtp_dns_res_opt;
char *var_psc_dnlist_action;
char *var_smtpd_tls_loglevel;
char *var_tlsp_clnt_logparam;
