int var_tlsp_clnt_scert_vd;
int var_psc_pre_queue_limit;
int var_smtpd_junk_cmd_limit;
int var_smtpd_hard_erlim;
int var_verify_poll_count;
int var_tlsp_tls_ccert_vd;
int var_smtpd_soft_erlim;
int var_psc_cconn_limit;
int var_psc_dnsbl_althresh;
int var_psc_post_queue_limit;
