int var_smtpd_sasl_resp_limit;
int var_map_reject_code;
int var_smtp_tls_scert_vd;
int var_smtp_reuse_count;
int var_min_delivery_slots;
int var_vrfy_pend_limit;
int var_init_dest_concurrency;
int var_unv_rcpt_dcode;
int var_unv_from_rcode;
int var_unk_client_code;
int var_smtp_line_limit;
int var_dest_con_limit;
int var_dup_filter_limit;
int var_smtpd_crate_limit;
int var_mul_rcpt_code;
int var_xport_refill_limit;
int var_psc_cmd_count;
int var_dest_rcpt_limit;
int var_smtpd_cipv6_prefix;
int var_smtpd_cauth_limit;
int var_unv_rcpt_rcode;
int var_bad_name_code;
int var_bounce_limit;
int var_virt_alias_code;
int var_smtpd_rcpt_overlim;
int var_reject_code;
int var_maps_rbl_code;
int var_smtp_min_data_rate;
int var_smtpd_cconn_limit;
int var_psc_dnsbl_thresh;
int var_qmgr_rcpt_limit;
int var_local_rcpt_code;
int var_delivery_slot_cost;
int var_local_rcpt_lim;
int var_virt_expan_limit;
int var_qmgr_fudge;
int var_smtpd_min_data_rate;
int var_smtpd_cmail_limit;
int var_smtpd_forbid_bare_lf_code;
int var_smtp_mxsess_limit;
int var_virt_minimum_uid;
int var_tls_rand_bytes;
int var_plaintext_code;
int var_relay_rcpt_code;
int var_unv_from_dcode;
int var_smtpd_rcpt_limit;
int var_qattr_count_limit;
int var_smtpd_policy_try_limit;
int var_smtpd_policy_req_limit;
int var_smtpd_cipv4_prefix;
int var_virt_mailbox_code;
int var_smtpd_hist_thrsh;
int var_non_fqdn_code;
int var_map_defer_code;
int var_unk_addr_code;
int var_unk_name_code;
int var_smtp_mxaddr_limit;
int var_body_check_len;
int var_smtpd_crcpt_limit;
int var_relay_code;
int var_delivery_slot_discount;
int var_delivery_slot_loan;
int var_xport_rcpt_limit;
int var_qmgr_msg_rcpt_limit;
int var_conc_cohort_limit;
int var_local_con_lim;
int var_qmgr_active_limit;
int var_virt_recur_limit;
int var_smtpd_cntls_limit;
int var_defer_code;
int var_stack_rcpt_limit;
int var_hopcount_limit;
int var_smtpd_tls_ccert_vd;
int var_virt_addrlen_limit;
